local E, L, V, P, G = unpack(ElvUI);
local mPlugin = "mMediaTag"
local DT = E:GetModule("DataTexts");
local addon, ns = ...
local displayString = ""
local mText = L["Professions"]

local menuFrame = CreateFrame("Frame", "mProfessionMenu", E.UIParent)
menuFrame:SetTemplate("Transparent", true)

local prof1, prof2, archaeology, fishing, cooking, firstAid = nil
local name, icon, skillLevel, maxSkillLevel, skillLine, skillModifier, specializationIndex, specializationOffset = nil
local mI = 0
local menuList = {}

local function getProf(index)
	local mskillModifier = 0
	local mIcon = E.db[mPlugin].mProfIcon
	name, icon, skillLevel, maxSkillLevel, _, _, skillLine, skillModifier, _, _ = GetProfessionInfo(index)
	
	if skillModifier ~= 0 then mskillModifier = " +" .. skillModifier else mskillModifier = "" end
	
	if mIcon then
		if (skillLevel == maxSkillLevel) then
			return format ("|T"..icon..":0|t  " .. name .. " (" .. ns.mColor6 .. '' .. skillLevel .. "|r" .. ns.mColor6 .. mskillModifier .. "|r)")
		else
			return format ("|T"..icon..":0|t  " .. name .. " (" .. ns.mColor2 .. '' .. skillLevel .. "|r" .. ns.mColor6 .. mskillModifier .. "/" .. ns.mColor7 .. '' .. maxSkillLevel .. "|r)")
		end
	else
		if (skillLevel == maxSkillLevel) then
			return format (name .. " (" .. ns.mColor6 .. '' .. skillLevel .. "|r" .. ns.mColor6 .. mskillModifier .. "|r)")
		else
			return format (name .. " (" .. ns.mColor2 .. '' .. skillLevel .. "|r" .. ns.mColor6 .. mskillModifier .. "/" .. ns.mColor7 .. '' .. maxSkillLevel .. "|r)")
		end
	end
 end
 
local function castProf(index)
	name, icon, skillLevel, maxSkillLevel, _, _, skillLine, skillModifier, specializationIndex, specializationOffset  = GetProfessionInfo(index)

	if skillLine == 182 then
		CastSpellByID(193290)
	elseif skillLine == 186 then
		CastSpellByID(2656)
	elseif skillLine == 202 then
		if specializationOffset == 1 then
			CastSpellByID(20219)
		else
			-- CastSpellByID(4036)
			-- CastSpellByID(4037)
			-- CastSpellByID(51306)
			-- CastSpellByID(30350)
			-- CastSpellByID(12656)
			-- CastSpellByID(4038)
			CastSpellByName(name)
		end
	elseif skillLine == 356 then
		CastSpellByID(271990)
	elseif skillLine == 393 then
		CastSpellByID(194174)
	else
		CastSpellByName(name)
	end
end

local function setupList(index, color, text, title, spell)
	if spell == nil then
		tinsert(menuList, index, {text = color .. text, isTitle = title, func = function() end})
	else
		tinsert(menuList, index, {text = color .. text, isTitle = title, func = function() castProf(spell) end})
	end
end

local function setupProf()
	prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
	mI = 1
	menuList = {}
	
	if (prof1 == nil) and (prof2 == nil) then
		setupList(mI, ns.mColor5, L["No Main Professions|r"], true, nil)
		mI = mI + 1
	else
		setupList(mI, ns.mColor2, L["Main Professions|r"], true, nil)
		mI = mI + 1
		
		if prof1 ~= nil then
			setupList(mI, ns.mColor1, getProf(prof1), false, prof1)
			mI = mI + 1
		end
	
		if prof2 ~= nil then
			setupList(mI, ns.mColor1, getProf(prof2), false, prof2)
			mI = mI + 1
		end
	end
	
	setupList(mI, ns.mColor1, "|r", true, nil)
	mI = mI + 1
	
	if  (archaeology == nil) and (cooking == nil) and (firstAid == nil) and (fishing == nil)  then
		setupList(mI, ns.mColor5, L["No Secondary Professions|r"], true, nil)
		mI = mI + 1
	else
		setupList(mI, ns.mColor2, L["Secondary Professions|r"], true, nil)
		mI = mI + 1
		
		if archaeology ~= nil then
			setupList(mI, ns.mColor1, getProf(archaeology), false, archaeology)
			mI = mI + 1
		end
		
		if cooking ~= nil then
			setupList(mI, ns.mColor1, getProf(cooking), false, cooking)
			mI = mI + 1
		end
		
		if firstAid ~= nil then
			setupList(mI, ns.mColor1, getProf(firstAid), false, firstAid)
			mI = mI + 1
		end
		
		if fishing ~= nil then
			setupList(mI, ns.mColor1, getProf(fishing), false, fishing)
			mI = mI + 1
		end
	end
end

local function setupTooltip ()
	DT.tooltip:AddLine(mText)
end

local function OnClick(self)
	local mProfSetting = E.db[mPlugin].mProfUp
	setupProf()
	mDropDown(menuList, menuFrame, mProfSetting, 200)
end
	
local function OnEnter(self)
	DT:SetupTooltip(self)
	setupTooltip()
	DT.tooltip:Show()
end

local function OnEvent(self, event, unit)
	self.text:SetFormattedText(displayString, mText)
end

local function OnLeave(self)
	DT.tooltip:Hide()
end

local function ValueColorUpdate(hex)
	displayString = strjoin("", hex, "%s|r")

	if lastPanel ~= nil then
		OnEvent(lastPanel, 'ELVUI_COLOR_UPDATE')
	end
end
E.valueColorUpdateFuncs[ValueColorUpdate] = true

DT:RegisterDatatext('Professions', nil, OnEvent, nil, OnClick, OnEnter, OnLeave, mText)